<?php
/**
 * @package Cluster
 */

/**
 * Return an object containing an array of CSS rules.
 *
 * This function behaves differently depending on which parameters are given.
 * - With no parameters, it returns the rules for all the CSS-related settings using their current stored values.
 * - With an array of setting IDs, it returns the rules for those specific settings, using their current stored values.
 * - With both an array of setting IDs and a corresponding array of values, it will return the rules for those specific settings, using the given values (sanitized).
 *
 * @since 1.0.0.
 *
 * @param  array $setting_ids       An optional array of setting IDs.
 * @param  array $preview_values    An optional array of values for the settings.
 *
 * @return mixed|CLUSTER_CSS|void
 */
function cluster_get_css_rules( $setting_ids = array(), $preview_values = array() ) {
	$settings      = new CLUSTER_Settings();
	$css           = new CLUSTER_CSS();
	$setting_pairs = array();

	// Default array of settings.
	if ( empty( $setting_ids ) ) {
		$setting_ids = array(
			'font-body',
			'font-headers',
			'style_background_color',
			'style_accent_color',
			'site_background_color',
			'site_background_opacity',
			'portfolio_background_color',
			'portfolio_background_opacity',
		);
	}

	// Use the saved setting values if no preview values are given.
	if ( empty( $preview_values ) ) {
		foreach ( (array) $setting_ids as $id ) {
			$setting_pairs[ $id ] = $settings->get_value( $id );
		}
	} else {
		$setting_pairs = array_combine( (array) $setting_ids, (array) $preview_values );
		array_walk( $setting_pairs, array( $settings, 'sanitize_value' ) );
	}

	// Iterate through each setting.
	foreach ( $setting_pairs as $setting_id => $sanitized_value ) {
		/**
		 * Filter to override the CSS rules for a particular setting.
		 *
		 * To override the CSS rules for a setting, create a function that accepts all three
		 * of the parameters of this filter. Use the CLUSTER_CSS methods in the $css parameter
		 * to add custom rules, then return $css instead of $override.
		 *
		 * Example:
		 * function child_css_override_titlecolor( $override, $setting_id, $css ) {
		 *     if ( 'color-site-title' === $setting_id ) {
		 *         $css->add( array(
		 *             'selectors'    => array(
		 *                 '.site-title', '.site-title a',
		 *             ),
		 *             'declarations' => array(
		 *                 'background-color' => $settings->get_value( $setting_id ),
		 *             ),
		 *         ) );
		 *         return $css;
		 *     }
		 *
		 *     return $override;
		 * }
		 * add_filter( 'cluster_css_rule_override', 'child_css_override_titlecolor', 10, 4 );
		 *
		 * To add rules for a setting not in the defaults list, or a new setting, use the 'custom-settings'
		 * value for the setting ID, like this:
		 *
		 *     if ( 'custom-settings' === $setting_id ) {
		 *         // Rules go here.
		 *     }
		 *
		 * @since 1.0.0.
		 *
		 * @param bool|CLUSTER_CSS    $override           Value determines whether setting CSS rules get overridden.
		 * @param string               $setting_id         The ID of the setting to override.
		 * @param mixed                $sanitized_value    The sanitized value currently assigned to the setting.
		 * @param CLUSTER_CSS         $css                The instance of CLUSTER_CSS.
		 */
		$override = apply_filters( 'cluster_css_rule_override', false, $setting_id, $sanitized_value, $css );
		if ( false !== $override && 'CLUSTER_CSS' === get_class( $override ) ) {
			$css = $override;
			continue;
		}

		switch ( $setting_id ) {
			case 'font-body' :
				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'font-family' => "'$sanitized_value'",
					),
				) );
				break;
			case 'font-headers' :
				$css->add( array(
					'selectors'    => array(
						'.heading',
						'h1,h2,h3,h4,h5,h6',
						'.custom-background-content h2',
					),
					'declarations' => array(
						'font-family' => "'$sanitized_value'",
					),
				) );
				break;
			case 'style_background_color' :
				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'background-color' => $sanitized_value,
					),
				) );
				break;
			case 'style_accent_color' :
				$css->add( array(
					'selectors'    => array(
						'a',
						'.stag-twitter a',
						'.entry-content h1',
						'.entry-content h2',
						'.entry-content h3',
						'.entry-content h4',
						'.entry-content h5',
						'.entry-content h6',
						'.footer .twitter-feeds li a',
					),
					'declarations' => array(
						'color' => $sanitized_value,
					),
				) );
				$css->add( array(
					'selectors'    => array(
						'.accent-background',
						'.more-link',
						'.page-numbers',
						'.nav-links > div',
						'input[type=submit]',
						'button',
						'.button',
						'.instagram-follow-link',
					),
					'declarations' => array(
						'background-color' => $sanitized_value,
					),
				) );
				break;
			case 'site_background_color' :
				$css->add( array(
					'selectors'    => array(
						'.custom-background.universal',
					),
					'declarations' => array(
						'background-color' => $sanitized_value . ' !important',
					),
				) );
				break;
			case 'site_background_opacity' :
				$css->add( array(
					'selectors'    => array(
						'.custom-background.universal .backstretch',
					),
					'declarations' => array(
						'opacity' => $sanitized_value / 100,
					),
				) );
				break;
			case 'portfolio_background_color' :
				$css->add( array(
					'selectors'    => array(
						'.single-portfolio .custom-background.universal',
						'.page-template-template-portfolio-php .custom-background.universal',
					),
					'declarations' => array(
						'background-color' => $sanitized_value . ' !important',
					),
				) );
				break;
			case 'portfolio_background_opacity' :
				$css->add( array(
					'selectors'    => array(
						'.single-portfolio .custom-background.universal .backstretch',
						'.page-template-template-portfolio-php .custom-background.universal .backstretch',
					),
					'declarations' => array(
						'opacity' => $sanitized_value / 100,
					),
				) );
				break;
		}
	}

	return $css;
}

/**
 * Output the theme CSS rules as an inline style element.
 *
 * Used in the Customizer preview pane.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function cluster_inline_css_rules() {
	// Get the rules.
	$css = cluster_get_css_rules();

	// Build the rules.
	$output = $css->build();

	// Output the rules.
	if ( ! empty( $output ) ) {
		echo "\n<style type=\"text/css\" id=\"cluster-inline-css\">\n";
		echo  $output; // phpcs:ignore
		echo "\n</style>\n";
	}
}

if ( is_customize_preview() ) {
	// Only add the inline rules in the Customizer preview pane.
	add_action( 'wp_head', 'cluster_inline_css_rules', 99 );
}

/**
 * Add the theme CSS rules to the content editor.
 *
 * @since 1.0.0.
 *
 * @param  string $stylesheets    The comma-separated string of stylesheet URLs.
 *
 * @return string                 The modified string of stylesheet URLs.
 */
function cluster_editor_css_rules( $stylesheets ) {
	$stylesheets .= ',' . add_query_arg( 'action', 'cluster-css-rules', admin_url( 'admin-ajax.php' ) );
	return $stylesheets;
}

add_filter( 'mce_css', 'cluster_editor_css_rules', 99 );

/**
 * Generates the theme CSS as an Ajax response.
 *
 * @since  1.0.0.
 *
 * @return void
 */
function cluster_ajax_css_rules() {
	// Make sure this is an Ajax request.
	if ( ! defined( 'DOING_AJAX' ) || true !== DOING_AJAX ) {
		return;
	}

	/**
	 * Filter whether the dynamic stylesheet will send headers telling the browser
	 * to cache the request. Set to false to turn off these headers.
	 *
	 * @since 1.0.0.
	 *
	 * @param bool    $cache_headers
	 */
	if ( true === apply_filters( 'cluster_stylesheet_cache_headers', true ) ) {
		// Set headers for caching
		// @link http://stackoverflow.com/a/15000868
		// @link http://www.mobify.com/blog/beginners-guide-to-http-cache-headers/
		$expires = HOUR_IN_SECONDS;
		header( 'Pragma: public' );
		header( 'Cache-Control: private, max-age=' . $expires );
		header( 'Expires: ' . gmdate( 'D, d M Y H:i:s', time() + $expires ) . ' GMT' );
	}

	// Set header for content type.
	header( 'Content-type: text/css' );

	// Get the rules.
	$css = cluster_get_css_rules();

	// Echo the rules.
	echo $css->build(); // phpcs:ignore

	// End the Ajax response.
	die();
}

add_action( 'wp_ajax_cluster-css-rules', 'cluster_ajax_css_rules' );
add_action( 'wp_ajax_nopriv_cluster-css-rules', 'cluster_ajax_css_rules' );

/**
 * Returns the theme CSS as a JSON object.
 *
 * Used for asynchronous previewing in the Customizer.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function cluster_ajax_css_json() {
	// Make sure this is an Ajax request.
	if ( ! defined( 'DOING_AJAX' ) || true !== DOING_AJAX ) {
		return;
	}

	$settings = new CLUSTER_Settings();

	// Get post parameters.
	$setting_ids = array();
	if ( isset( $_POST['sid'] ) ) {
		$setting_ids = array_map( 'sanitize_title_with_dashes', (array) $_POST['sid'] );
	}
	$values = array();
	if ( isset( $_POST['val'] ) ) {
		foreach ( (array) $_POST['val'] as $index => $raw_value ) {
			$values[ $index ] = $settings->sanitize_value( $raw_value, $setting_ids[ $index ] );
		}
	}

	// Generate the rules.
	$css = cluster_get_css_rules( $setting_ids, $values );

	// Send response.
	if ( isset( $css->data ) && is_array( $css->data ) && ! empty( $css->data ) ) {
		wp_send_json_success( $css->data );
	} else {
		wp_send_json_error( $css );
	}
}

add_action( 'wp_ajax_cluster-css-json', 'cluster_ajax_css_json' );

/**
 * Saves a version number (Unix timestamp) when Customizer settings are saved.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function cluster_customize_save_version() {
	set_theme_mod( 'version', time() );
}

add_action( 'customize_save_after', 'cluster_customize_save_version' );
